" Vim syntax file for Asterix programs
" Language:     Asterix
" Author:       Pim Goossens <ppgdev@xs4all.nl>
" Copyright:    None. This file ("asterix-syntax.vim") is in the public domain.

" To use:       cp asterix-syntax.vim ~/.vim/syntax/asterix.vim

" Quit when a (custom) syntax file was already loaded
if exists("b:current_syntax")
    finish
endif

syn keyword     asterixTodo     contained TODO FIXME
syn match       asterixComment  "#.*" contains=asterixTodo

syn keyword     asterixOuterKW  entry
syn keyword     asterixItemKW   contained alloca
syn keyword     asterixExprKW   contained syscall

syn match       asterixMatchOp  "[!&()*.<=>?^|]"

syn match       asterixName     "[A-Za-z][0-9A-Za-z_]*"

syn region      asterixOutput   matchgroup=asterixOutBrkt start="<" end=">" contains=asterixTodo,asterixComment,asterixExprKW,asterixName,asterixChar,asterixNumber,asterixMemExpr,asterixBin,asterixString
syn match       asterixOutName  contained containedin=asterixOutput "[A-Za-z][0-9A-Za-z_]*"

syn region      asterixDef      matchgroup=asterixDefStart start="[A-Za-z][0-9A-Za-z_]*\s*\(([[:space:],0-9A-Za-z_]*)\s*\)\?\(:[[:space:],0-9A-Za-z\[\]_]*\(:[[:space:],0-9A-Za-z\[\]_]*\)\?\)\?\s*=" matchgroup=asterixDefEnd end=";" contains=asterixTodo,asterixComment,asterixItemKW,asterixMatchOp,asterixOutput,asterixName,asterixChar,asterixNumber,asterixRange,asterixBin,asterixExpr,asterixString
" syn match       asterixDefStart contained "[A-Za-z][0-9A-Za-z_]*\s*\(([[:space:],0-9A-Za-z_]*)\s*\)\?\(:[[:space:],0-9A-Za-z\[\]_]*\(:[[:space:],0-9A-Za-z\[\]_]*\)\?\)\?\s*=" contains=asterixFunc,asterixVarSep,asterixNumber
syn match       asterixFunc     "\([^;]\s*\)\@8<![A-Za-z][0-9A-Za-z_]*\(\s*[():=]\)\@="
syn match       asterixArgs     "([[:space:],0-9A-Za-z_]*)"
syn match       asterixVarSep   ":"

syn match       asterixChar     "'\\x..'\|'\\.'\|'[^\\]'" contains=asterixEscChar
syn match       asterixNumber   "\(-\|\<\)\(0x[0-9A-Fa-f]\+\|[0-9]\+\)\>"

syn match       asterixRange    "\[\(\\x..\|\\.\|[^\\]\)-\(\\x..\|\\.\|[^\\]\)\]" contains=asterixEscChar
syn region      asterixMemExpr  matchgroup=asterixMemRef start="\[s\?[bwdq]:" end="\]"
syn match       asterixBin      "%[bwdqs]:"

syn region      asterixExpr     start="{" end="}" contains=asterixTodo,asterixComment,asterixExprKW,asterixName,asterixChar,asterixNumber,asterixMemExpr,asterixString

syn match       asterixString   "\"\([^"\\]*\|\\x..\|\\.\)*\"" contains=asterixEscChar
syn match       asterixEscChar  contained "\\['"\\0abtnvfre]"
syn match       asterixEscChar  contained "\\x[0-9A-Fa-f][0-9A-Fa-f]"

hi def link     asterixTodo     Todo
hi def link     asterixComment  Comment
hi def link     asterixOuterKW  Keyword
hi def link     asterixItemKW   Keyword
hi def link     asterixExprKW   Keyword
hi def link     asterixMatchOp  Operator
hi def link     asterixDefEnd   Operator
hi def link     asterixFunc     Identifier
hi def link     asterixVarSep   SpecialChar
hi def link     asterixOutBrkt  SpecialChar
hi def link     asterixOutName  SpecialChar
hi def link     asterixChar     Character
hi def link     asterixNumber   Number
hi def link     asterixRange    Constant
hi def link     asterixMemRef   Type
hi def link     asterixBin      SpecialChar
hi def link     asterixString   String
hi def link     asterixEscChar  SpecialChar

" vim: sw=4 ts=4 et
